#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdstatold import sdstatold
from task_sdstatold import casalog

class sdstatold_pg_:
    __name__ = "sdstatold"

    def __init__(self) :
       self.__bases__ = (sdstatold_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,antenna=None,fluxunit=None,telescopeparm=None,specunit=None,restfreq=None,frame=None,doppler=None,scanlist=None,field=None,iflist=None,pollist=None,masklist=None,invertmask=None,interactive=None,outfile=None,format=None,overwrite=None,):

        """ASAP SD task: list statistics of spectral region
        Keyword arguments:
        infile -- name of input SD dataset
                default: none - must input file name
                example: 'mysd.asap'
                         See sdcalold for allowed formats.
        antenna -- antenna name or id (only effective for MS input). 
        fluxunit -- units for line flux
                options: (str) 'K','Jy',''
                default: '' (keep current fluxunit)
                WARNING: For GBT data, see description below.
            >>> fluxunit expandable parameter
                 telescopeparm -- the telescope characteristics
                        options: (str) name or (list) list of gain info
                        default: '' (none set)
                        example: if telescopeparm='', it tries to get the telescope
                                 name from the data.
                                 Full antenna parameters (diameter,ap.eff.) known
                                 to ASAP are
                                 'ATPKSMB', 'ATPKSHOH', 'ATMOPRA', 'DSS-43',
                                 'CEDUNA','HOBART'. For GBT, it fixes default fluxunit
                                 to 'K' first then convert to a new fluxunit.
                                 telescopeparm=[104.9,0.43] diameter(m), ap.eff.
                                 telescopeparm=[0.743] gain in Jy/K
                                 telescopeparm='FIX' to change default fluxunit
                                 see description below

        specunit -- units for spectral axis
                options: (str) 'channel','km/s','GHz','MHz','kHz','Hz',''
                default: '' (=current)
            >>> specunit expandable parameter
                 restfreq -- rest frequency
                         default: '' (use current setting)
                         example: 4.6e10 (float value in Hz),
                                  '46GHz' (string with unit),
                                  ['345.8GHz', 347.0e9, 356.7e9] (for each IF)
                                  [{'name':'CO','value':345e9}] (a value with name)
        frame -- frequency frame for spectral axis
                options: (str) 'LSRK','REST','TOPO','LSRD','BARY',
                         'GEO','GALACTO','LGROUP','CMB'
                default: currently set frame in scantable
                WARNING: frame='REST' not yet implemented
        doppler -- doppler mode
                options: (str) 'RADIO','OPTICAL','Z','BETA','GAMMA'
                default: currently set doppler in scantable
        scanlist -- list of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field, iflist, and pollist
        field -- selection string for selecting scans by name
                default: '' (no name selection)
                example: 'FLS3a*'
                this selection is in addition to scanlist, iflist, and pollist
        iflist -- list of IF id numbers to select
                default: [] (use all IFs)
                example: [15]
                this selection is in addition to field, scanlist, and pollist
        pollist -- list of polarization id numbers to select
                default: [] (use all pols)
                example: [1]
                this selection is in addition to field, scanlist, and iflist
        masklist -- list of mask regions to INCLUDE in stats
                default: [] (whole spectrum)
                example: [4000,4500] for one region
                         [[1000,3000],[5000,7000]]
                         these must be pairs of [lo,hi] boundaries
        invertmask -- invert mask (EXCLUDE masklist instead)
                options: (bool) True,False
                default: false
        interactive -- determines interactive masking
                options: (bool) True,False
                default: False
                example: interactive=True allows adding and deleting mask 
                         regions by drawing rectangles on the plot with mouse. 
                         Draw a rectangle with LEFT-mouse to ADD the region to 
                         the mask and with RIGHT-mouse to DELETE the region. 
        outfile -- name of output file for line statistics
                default: '' (no output statistics file)
                example: 'stat.txt'
        format -- format string to print statistic values
                default: '3.3f'
        overwrite -- overwrite the statistics file if already exists 
                options: (bool) True,False
                default: False

        -------------------------------------------------------------------
                Returns: a Python dictionary of line statistics
                   keys: 'rms','stddev','max','min','max_abscissa',
                         'min_abscissa','sum','median','mean','totint','eqw'
                example: xstat=sdstatold(); print "rms = ",xstat['rms']
                         these can be used for testing in scripts or
                         for regression

                         'max_abscissa' and 'min_abscissa' refer to the abscissa
                         (channel/frequency/velocity) of max and min intensity.
                         'totint' is the integrated intensity (sum*dx)
                         where dx is the abscissa interval in 'specunit'.
                         'eqw' is equivalent width (totint/mag) where mag
                         is either max or min depending on which has
                         greater magnitude. 
                         Note that 'max_abscissa', 'min_abscissa', 'totint' 
                         and 'eqw' are quantities (python dictionaries with
                         keys, 'unit' and 'value').


       DESCRIPTION:

        Task sdstatold computes basic statistics (rms,mean,median,sum)
        for single-dish spectra.  It assumes that the spectra have
        been calibrated.  Furthermore, it assumes that any
        time and channel averaging/smoothing has also already been done as
        there are no controls for these. 

        Note that multiple scans and IFs can in principle be handled, but
        we recommend that you use scanlist, field, iflist, and pollist to give 
        a single selection for each run.

        WARNING: If you do have multiple scantable rows, then the returning 
        values will be lists.

        ASAP recognizes the data of the "AT" telescopes, but currently
        does not know about the GBT or any other telescope. This task
        does know about GBT. Telescope name is obtained from the data.
        If you wish to change the fluxunit (see below), and telescopeparm='',
        for the AT telescopes it will use internal telescope parameters for
        flux conversion. For GBT, it will use an approximate aperture
        efficiency conversion.  If you give telescopeparm a list, then
        if the list has a single float it is assumed to be the gain in Jy/K,
        if two or more elements they are assumed to be telescope diameter (m)
        and aperture efficiency respectively.

        WARNING for the GBT raw SDFITS format data as input:
        SDtasks are able to handle GBT raw SDFITS format data since the 
        data filler is available. However, the functionality is not well 
        tested yet, so that there may be unknown bugs.  

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdstatold'
        myf['taskname'] = 'sdstatold'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['fluxunit'] = fluxunit = myf['fluxunit']
            myparams['telescopeparm'] = telescopeparm = myf['telescopeparm']
            myparams['specunit'] = specunit = myf['specunit']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['frame'] = frame = myf['frame']
            myparams['doppler'] = doppler = myf['doppler']
            myparams['scanlist'] = scanlist = myf['scanlist']
            myparams['field'] = field = myf['field']
            myparams['iflist'] = iflist = myf['iflist']
            myparams['pollist'] = pollist = myf['pollist']
            myparams['masklist'] = masklist = myf['masklist']
            myparams['invertmask'] = invertmask = myf['invertmask']
            myparams['interactive'] = interactive = myf['interactive']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['format'] = format = myf['format']
            myparams['overwrite'] = overwrite = myf['overwrite']

        if type(scanlist)==int: scanlist=[scanlist]
        if type(iflist)==int: iflist=[iflist]
        if type(pollist)==int: pollist=[pollist]
        if type(masklist)==int: masklist=[masklist]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['fluxunit'] = fluxunit
        mytmp['telescopeparm'] = telescopeparm
        mytmp['specunit'] = specunit
        mytmp['restfreq'] = restfreq
        mytmp['frame'] = frame
        mytmp['doppler'] = doppler
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['iflist'] = iflist
        mytmp['pollist'] = pollist
        mytmp['masklist'] = masklist
        mytmp['invertmask'] = invertmask
        mytmp['interactive'] = interactive
        mytmp['outfile'] = outfile
        mytmp['format'] = format
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdstatold.xml')

        casalog.origin('sdstatold')
        if not trec.has_key('sdstatold') or not casac.utils().verify(mytmp, trec['sdstatold']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdstatold           #####')
          casalog.post('')
          result = sdstatold(infile, antenna, fluxunit, telescopeparm, specunit, restfreq, frame, doppler, scanlist, field, iflist, pollist, masklist, invertmask, interactive, outfile, format, overwrite)
          casalog.post('')
          casalog.post('##### End Task: sdstatold           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdstatold', 'sdstatold.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdstatold'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdstatold', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['specunit']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['masklist']  = []
        a['invertmask']  = False
        a['interactive']  = False
        a['outfile']  = ''
        a['format']  = '3.3f'
        a['overwrite']  = False

        a['fluxunit'] = {
                    0:{'value':''}, 
                    1:odict([{'value':'K'}, {'telescopeparm':''}]), 
                    2:odict([{'value':'k'}, {'telescopeparm':''}]), 
                    3:odict([{'value':'Jy'}, {'telescopeparm':''}]), 
                    4:odict([{'value':'jy'}, {'telescopeparm':''}])}
        a['specunit'] = {
                    0:{'value':''}, 
                    1:{'value':'channel'}, 
                    2:odict([{'value':'km/s'}, {'restfreq':''}]), 
                    3:{'value':'GHz'}, 
                    4:{'value':'MHz'}, 
                    5:{'value':'kHz'}, 
                    6:{'value':'Hz'}}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdstatold', subkey=None):
        desc={'sdstatold': 'ASAP SD task: list statistics of spectral region',
               'infile': 'name of input SD dataset',
               'antenna': 'antenna name or id (only effective for MS input)',
               'fluxunit': 'units for line flux (K,Jy) (''=current)',
               'telescopeparm': 'param of telescope for flux conversion',
               'specunit': 'units for spectral axis (channel,km/s,GHz)',
               'restfreq': 'rest frequency (default unit: Hz)',
               'frame': 'frequency reference frame, e.g. LSRK (''=current)',
               'doppler': 'doppler convention, e.g. RADIO (''=current)',
               'scanlist': 'list of scans to use (e.g. [1,2,3,4])',
               'field': 'string for selection by source name',
               'iflist': 'list of IF ids to select (e.g. [0,1])',
               'pollist': 'list of polarization ids to select (e.g. [0,1])',
               'masklist': 'list of mask regions to INCLUDE in stats',
               'invertmask': 'invert mask (EXCLUDE masklist instead)',
               'interactive': 'determines interactive masking',
               'outfile': 'name of output file for line statistics',
               'format': 'format string to print statistic values',
               'overwrite': 'overwrite the statistics file if already exists',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['telescopeparm']  = ''
        a['specunit']  = ''
        a['restfreq']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['masklist']  = []
        a['invertmask']  = False
        a['interactive']  = False
        a['outfile']  = ''
        a['format']  = '3.3f'
        a['overwrite']  = False

        if a.has_key(paramname) :
	      return a[paramname]
sdstatold_pg = sdstatold_pg_()
